<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\FormHelperTrait;

/**
 * Form that displays all the config variables to edit them.
 */
class EnforcedConfigs extends FormBase {

  use FormHelperTrait;

  const FORM_ID = 'config_enforce_enforced_configs_form';

  protected $typeFilter;
  protected $nameFilter;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $type = 'all', string $partial = '') {
    $this->setSharedFormProperties($form, $form_state);
    $this->typeFilter = $type;
    $this->nameFilter = $partial;

    $this->addFilter();

    $this->form()['config_objects'] = [
      '#type' => 'table',
      '#header' => $this->getHeader(),
      '#rows' => $this->getRows(),
      '#empty' => $this->t('No config objects found'),
    ];

    $this->form()['#attached']['library'][] = 'config_enforce_devel/config-enforce-devel';
    $this->form()['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $this->form();
  }

  /**
   * {@inheritdoc}
   *
   * This provides an easy way for enforced configs to be enforced themselves.
   */
  public function getEditableConfigNames() {
    return ['config_enforce.enforced_configs'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);
    $redirect = Url::FromRoute('config_enforce_devel.enforced_configs', [
      'type' => Html::escape($this->formState()->getValue('type')),
      'partial' => Html::escape($this->formState()->getValue('name')),
    ]);
    $this->formState()->setRedirectUrl($redirect);
  }

  /**
   * Add a filter to top of form.
   */
  protected function addFilter() {
    $open = trim($this->typeFilter) != '' || trim($this->nameFilter) != '';
    $this->form()['filter'] = [
      '#type' => 'details',
      '#title' => $this->t('Filter enforced configs'),
      '#attributes' => ['class' => ['container-inline']],
      '#open' => $open,
    ];
    $this->form()['filter']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Config object name'),
      '#title_display' => 'invisible',
      '#default_value' => $this->nameFilter,
    ];
    $this->form()['filter']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Config type'),
      '#title_display' => 'invisible',
      '#options' => [
        'all' => 'All config objects',
        'enforced' => 'Only enforced config objects',
        'unenforced' => 'Only unenforced config objects',
      ],
      '#default_value' => $this->typeFilter,
    ];

    $this->form()['filter']['actions'] = ['#type' => 'actions'];
    $this->form()['filter']['actions']['show'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
  }

  /**
   * Return a table header render array.
   */
  protected function getHeader() {
    return [
      'name' => ['data' => $this->t('Name')],
      'module' => ['data' => $this->t('Module')],
      'directory' => ['data' => $this->t('Directory')],
      'settings' => ['data' => $this->t('Settings')],
    ];
  }

  /**
   * Return a table rows render array.
   */
  protected function getRows() {
    $destination = $this->getDestinationArray();

    $rows = [];
    foreach ($this->getConfigNames() as $config_name) {
      $settings = [
        'data' => [
          '#type' => 'link',
          '#title' => $this->t('Edit'),
          '#url' => Url::fromRoute('config_enforce_devel.config_edit', ['arg' => $config_name]),
          '#attributes' => [
            'class' => [
              'use-ajax',
              'button',
            ],
          ],
        ],
      ];

      $config_display_name = $config_name;
      $uri = ConfigEnforce::getUri($config_name);
      if (!empty($uri)) {
        $config_display_name = Link::fromTextAndUrl($config_name, Url::fromUri('internal:' . $uri));
      }

      $rows[] = [
        'class' => $this->getRowClasses($config_name),
        'data' => [
          'name' => $config_display_name,
          'module' => ConfigEnforce::getModule($config_name),
          'directory' => ConfigEnforce::getDirectory($config_name),
          'settings' => $settings,
        ],
      ];
    }

    return $rows;
  }

  protected function getRowClasses($config_name) {
    $classes = [];
    if (ConfigEnforce::isEnforced($config_name)) {
      $classes[] = 'enforced';
      $classes[] = 'level-' . ConfigEnforce::getLevel($config_name);
    }
    return $classes;
  }

  /**
   * Return a list of config names, properly filtered.
   */
  protected function getConfigNames() {
    switch ($this->typeFilter) {
      case 'enforced':
        return $this->getEnforcedConfigs();
      case 'unenforced':
        return $this->getUnenforcedConfigs();
      case 'all':
      default:
        return $this->getFilteredConfigs();
    }
  }

  /**
   * List all config object names filtered by partial name if any was provided.
   */
  protected function getFilteredConfigs() {
    return $this->configFactory()->listAll($this->nameFilter);
  }

  /**
   * List only enforced config object names.
   */
  protected function getEnforcedConfigs() {
    $names = $this->getFilteredConfigs();
    $enforced = [];
    foreach (ConfigEnforce::getEnforcedConfigs() as $name => $info) {
      if (in_array($name, $names)) {
        $enforced[] = $name;
      }
      // @TODO figure out what to do with missing/deleted enforced configs.
    }
    return $enforced;
  }

  /**
   * List only unenforced config object names.
   */
  protected function getUnenforcedConfigs() {
    return array_diff($this->getFilteredConfigs(), $this->getEnforcedConfigs());
  }

}
