<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\Core\Config\InstallStorage;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\config_enforce_devel\Form\DevelFormHelperTrait;

/**
 * Select which config objects to import.
 */
class ImportForm extends ConfirmFormBase {

  use DevelFormHelperTrait;

  const FORM_ID = 'config_enforce_devel_import_form';

  protected $step = 1;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    // When this is the confirmation step fall through to the confirmation form.
    if ($this->step === 2) {
      return parent::buildForm($this->form(), $this->formState());
    }

    $this->form()['import'] = [
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#collapsible' => FALSE,
      '#title' => $this->t('Import'),
      '#description' => $this->t('TBD.'),
    ];

    $this->form()['#attached']['library'][] = 'config_enforce_devel/config-enforce-devel';
    $this->form()['#attached']['library'][] = 'core/drupal.dialog.ajax';

    $this->addLevel();
    $this->addModules();
    $this->addFormActions();

    return $this->form();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    if ($this->step === 1) {
      // Rebuild form to trigger confirmation behaviour.
      $this->formState()->setRebuild();
      $this->step = 2;
      return;
    }
    $this->importConfigEnforceSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return 'Confirm import of config enforcement settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return Url::fromRoute('config_enforce_devel.import');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('Create enforcement settings for the following config objects: @list', [
      '@list' => $this->getImportsHtmlList(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return 'Confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelText() {
    return 'Cancel';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormName() {
    return 'config_enforce_devel_import_confirm_form';
  }

  /**
   * Add select list to choose the enforcement level to apply to imports.
   */
  protected function addLevel() {
    $this->form()['import']['level'] = [
      '#type' => 'select',
      '#title' => $this->t('Enforcement level'),
      '#description' => "The level of enforcement to apply to imported config enforce settings.",
      '#options' => ConfigEnforce::getLevels(),
      '#default_value' => ConfigEnforce::CONFIG_ENFORCE_NOSUBMIT,
    ];
  }

  /**
   * Add checkboxes for selected modules.
   */
  protected function addModules() {
    $this->form()['import']['modules'] = [
      '#type' => 'fieldgroup',
      '#tree' => TRUE,
      '#title' => $this->t('Modules'),
    ];

    foreach ($this->getAvailableModules() as $module => $name) {
      $this->addModule($module);
    }
  }

  /**
   * Add a checkbox for a single module.
   */
  protected function addModule($module) {
    $this->form()['import']['modules'][$module]['import'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('%module (@module)', [
        '%module' => $this->getModuleName($module),
        '@module' => $module,
      ]),
      '#attributes' => [
        'name' => $module,
      ]
    ];
    $this->addModuleConfigDirectories($module);
  }

  /**
   * Add checkboxes for config directories.
   */
  protected function addModuleConfigDirectories($module) {
    $this->form()['import']['modules'][$module]['config_dirs'] = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="' . $module . '"]' => ['checked' => TRUE],
        ],
      ],
    ];
    foreach ($this->getAvailableDirectories() as $directory => $name) {
      $this->addModuleConfigDirectory($module, $directory);
    }
  }

  /**
   * Add a checkbox for a single config directory.
   */
  protected function addModuleConfigDirectory($module, $directory) {
    $config_dir = $module . '-' . $directory;
    $this->form()['import']['modules'][$module]['config_dirs'][$directory]['import'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('%config_dir (@config_dir)', [
        '%config_dir' => $this->getDirectoryLabel($directory),
        '@config_dir' => $directory,
      ]),
      '#attributes' => [
        'name' => $config_dir,
      ]
    ];
    $this->addModuleConfigs($module, $directory);
  }

  /**
   * Add a checkboxes for config objects from a config directory.
   */
  protected function addModuleConfigs($module, $directory) {
    $config_dir = $module . '-' . $directory;
    $this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'] = [
      '#type' => 'container',
      '#tree' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="' . $config_dir . '"]' => ['checked' => TRUE],
        ],
      ],
    ];
    foreach ($this->getModuleConfigNames($module, $directory) as $config_name) {
      $this->addModuleConfig($module, $directory, $config_name);
    }
  }

  /**
   * Add a checkbox for a single config object to enforce.
   */
  protected function addModuleConfig($module, $directory, $config_name) {
    $config_dir = $module . '-' . $directory;
    $this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'][$config_name] = [
      '#type' => 'checkbox',
      '#title' => $config_name,
      '#states' => [
        'checked' => [
          ':input[name="' . $config_dir . '"]' => ['checked' => TRUE],
        ],
      ],
    ];

    if (ConfigEnforce::isEnforced($config_name)) {
      $this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'][$config_name]['#title'] .= $this->t(' (already enforced)');
      $this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'][$config_name]['#disabled'] = TRUE;
      $this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'][$config_name]['#default_value'] = FALSE;
      unset($this->form()['import']['modules'][$module]['config_dirs'][$directory]['configs'][$config_name]['#states']);
    }
  }

  /**
   * Attach a submit button to the form.
   */
  protected function addFormActions() {
    $this->form()['actions'] = [
      '#type' => 'actions',
    ];
    $this->form()['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => 'Import',
      '#button_type' => 'primary',
    ];
  }

  /**
   * Return an array of configs to import with their respective settings.
   */
  protected function getConfigsToImport() {
    $import = $this->formState()->getValue('import');
    $enforced_configs = [];
    foreach ($import['modules'] as $module => $module_data) {
      if (empty($module_data['config_dirs'])) continue;
      foreach ($module_data['config_dirs'] as $config_dir => $config_dir_data) {
        if (empty($config_dir_data['configs'])) continue;
        foreach ($config_dir_data['configs'] as $config => $checked) {
          if (!$checked) continue;
          $enforced_configs[$config] = [
             'module' => $module,
             'directory' => $config_dir,
             'level' => $import['level'],
          ];
        }
      }
    }
    return $enforced_configs;
  }

  /**
   * Returns an HTML-formatted list of config enforcements to import.
   */
  protected function getImportsHtmlList() {
    return $this->renderHtmlList(array_keys($this->getConfigsToImport()));
  }

  /**
   * Create config enforcement settings for selected config objects.
   */
  protected function importConfigEnforceSettings() {
    $enforce = new ConfigEnforceDevel();
    foreach ($this->getConfigsToImport() as $config_name => $settings) {
      $enforce
        ->setConfig($config_name)
        ->setModule($settings['module'])
        ->setDirectory($settings['directory'])
        ->setLevel($settings['level'])
        ->save();

      $this->messenger()->addMessage(
        $this->t('Imported enforcement settings for: %config', [
          '%config' => $config_name,
        ])
      );
    }
  }

}
