<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\RedirectCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\config_enforce\Form\FormHelperTrait;

/**
 * Methods to simplify modal form handling.
 */
trait ModalFormHelperTrait {

  use FormHelperTrait;

  protected $modalWidth = '800';

  /**
   * Callback for opening the modal form.
   */
  public function openModalForm($arg = '') {
    $response = new AjaxResponse();

    // Get the modal form using the form builder.
    $modal_form = \Drupal::formBuilder()->getForm(get_called_class(), $arg);

    // Add an AJAX command to open a modal dialog with the form as the content.
    $response->addCommand(new OpenModalDialogCommand($this->getModalTitle($arg), $modal_form, ['width' => $this->getModalWidth()]));

    return $response;
  }

  /**
   * AJAX callback handler that displays any errors or a success message.
   */
  public function submitModalFormAjax(array $form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    $response = new AjaxResponse();

    // If there are any form errors, re-display the form.
    if ($this->formState()->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('#' . $this->getFormId(), $this->form()));
    }
    else {
      // @TODO Preserve filter parameters.
      $response->addCommand(new CloseModalDialogCommand('#' . $this->getFormId()));
      $response->addCommand(new RedirectCommand($this->getRedirectUrl()));
    }

    return $response;
  }

  /**
   * Return the URL to redirect to after saving the modal form.
   */
  protected function getRedirectUrl() {
    return Url::fromRoute(get_called_class()::REDIRECT_ROUTE)
      ->toString();
  }

  /**
   * Return a title for the modal.
   */
  protected function getModalTitle(string $arg) {
    return $this->t(get_called_class()::MODAL_TITLE, ['%arg' => $arg]);
  }

  /**
   * Return the desired width of the modal.
   */
  protected function getModalWidth() {
    return $this->modalWidth;
  }

  /**
   * Prepare this form to be displayed in a modal dialog.
   */
  protected function makeFormModal() {
    $this->form()['#prefix'] = "<div id='" . $this->getFormId() . "'>";
    $this->form()['#suffix'] = "</div>";
    $this->form()['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -10,
    ];
    $this->form()['actions'] = ['#type' => 'actions'];
    $this->form()['actions']['send'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#attributes' => [
        'class' => [
          'use-ajax',
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'submitModalFormAjax'],
        'event' => 'click',
      ],
    ];

    $this->form()['#attached']['library'][] = 'core/drupal.dialog.ajax';
  }

}
